@@ -121,6 +121,7 @@ Span* Span::CreateClientSpan(const std::string& full_method_name,
     span->_start_send_real_us = 0;
     span->_sent_real_us = 0;
     span->_next_client = NULL;
+    span->_client_list = NULL;
     span->_tls_next = NULL;
     span->_full_method_name = full_method_name;
     span->_info.clear();
@@ -129,8 +130,8 @@ Span* Span::CreateClientSpan(const std::string& full_method_name,
         span->_trace_id = parent->trace_id();
         span->_parent_span_id = parent->span_id();
         span->_local_parent = parent;
-        span->_next_client = parent->_next_client;
-        parent->_next_client = span;
+        span->_next_client = parent->_client_list;
+        parent->_client_list = span;
     } else {
         span->_trace_id = GenerateTraceId();
         span->_parent_span_id = 0;
@@ -140,6 +141,47 @@ Span* Span::CreateClientSpan(const std::string& full_method_name,
     return span;
 }
 
+Span* Span::CreateBthreadSpan(const std::string& full_method_name, 
+                              int64_t base_real_us) {
+    Span* parent = (Span*)bthread::tls_bls.rpcz_parent_span;
+    if (parent == NULL) {
+        return NULL;
+    }
+    Span* span = butil::get_object<Span>(Forbidden());
+    if (__builtin_expect(span == NULL, 0)) {
+        return NULL;
+    }
+    span->_log_id = 0;
+    span->_base_cid = INVALID_BTHREAD_ID;
+    span->_ending_cid = INVALID_BTHREAD_ID;
+    span->_type = SPAN_TYPE_BTHREAD;
+    span->_async = false;
+    span->_protocol = PROTOCOL_UNKNOWN;
+    span->_error_code = 0;
+    span->_request_size = 0;
+    span->_response_size = 0;
+    span->_base_real_us = base_real_us;
+    span->_received_real_us = 0;
+    span->_start_parse_real_us = 0;
+    span->_start_callback_real_us = 0;
+    span->_start_send_real_us = 0;
+    span->_sent_real_us = 0;
+    span->_next_client = NULL;
+    span->_client_list = NULL;
+    span->_tls_next = NULL;
+    span->_full_method_name = full_method_name;
+    span->_info.clear();
+
+    span->_trace_id = parent->trace_id();
+    span->_parent_span_id = parent->span_id();
+    span->_local_parent = parent;
+    span->_next_client = parent->_client_list;
+    parent->_client_list = span;
+
+    span->_span_id = GenerateSpanId();
+    return span;
+}
+
 inline const std::string& unknown_span_name() {
     // thread-safe in gcc.
     static std::string s_unknown_method_name = "unknown_method";
@@ -173,6 +215,7 @@ Span* Span::CreateServerSpan(
     span->_start_send_real_us = 0;
     span->_sent_real_us = 0;
     span->_next_client = NULL;
+    span->_client_list = NULL;
     span->_tls_next = NULL;
     span->_full_method_name = (!full_method_name.empty() ?
                                full_method_name : unknown_span_name());
@@ -195,15 +238,20 @@ void Span::ResetServerSpanName(const std::string& full_method_name) {
 
 void Span::destroy() {
     EndAsParent();
-    Span* p = _next_client;
-    while (p) {
-        Span* p_next = p->_next_client;
-        p->_info.clear();
-        butil::return_object(p);
-        p = p_next;
+    traversal(this, [](Span* r) {
+        r->_info.clear();
+        butil::return_object(r);
+    });
+}
+
+void Span::traversal(Span* r, const std::function<void(Span*)>& f) const {
+    if (r == NULL) {
+        return;
     }
-    _info.clear();
-    butil::return_object(this);
+    for (auto p = r->_client_list; p != NULL; p = p->_next_client) {
+        traversal(p, f);
+    }
+    f(r);
 }
 
 void Span::Annotate(const char* fmt, ...) {
@@ -243,8 +291,8 @@ void Span::AnnotateCStr(const char* info, size_t length) {
 
 size_t Span::CountClientSpans() const {
     size_t n = 0;
-    for (Span* p = _next_client; p; p = p->_next_client, ++n);
-    return n;
+    traversal(const_cast<Span*>(this), [&](Span*) { ++n; });
+    return n - 1;
 }
 
 int64_t Span::GetStartRealTimeUs() const {
@@ -577,9 +625,13 @@ leveldb::Status SpanDB::Index(const Span* span, std::string* value_buf) {
         value_proto.add_client_spans();
     }
     size_t i = 0;
-    for (const Span* p = span->_next_client; p; p = p->_next_client, ++i) {
+    span->traversal(const_cast<Span*>(span), [&](Span* p) {
+        if (span == p) {
+            return;
+        }
         Span2Proto(p, value_proto.mutable_client_spans(client_span_count - i - 1));
-    }
+        ++i;
+    });
     if (!value_proto.SerializeToString(value_buf)) {
         return leveldb::Status::InvalidArgument(
             leveldb::Slice("Fail to serialize RpczSpan"));
