@@ -55,14 +55,20 @@ int add(void* meta, bthread::TaskIterator<LongIntTask> &iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, single_thread) {
+void test_single_thread(bool use_pthread) {
     int64_t result = 0;
     int64_t expected_result = 0;
     stopped = false;
     bthread::ExecutionQueueId<LongIntTask> queue_id;
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
-                                                    add, &result));
+                                                add, &result));
     for (int i = 0; i < 100; ++i) {
         expected_result += i;
         ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, i));
@@ -75,6 +81,12 @@ TEST_F(ExecutionQueueTest, single_thread) {
     ASSERT_TRUE(stopped);
 }
 
+TEST_F(ExecutionQueueTest, single_thread) {
+    for (int i = 0; i < 2; ++i) {
+        test_single_thread(i);
+    }
+}
+
 class RValue {
 public:
     RValue() : _value(0) {}
@@ -105,14 +117,20 @@ int add(void* meta, bthread::TaskIterator<RValue> &iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, rvalue) {
+void test_rvalue(bool use_pthread) {
     int64_t result = 0;
     int64_t expected_result = 0;
     stopped = false;
     bthread::ExecutionQueueId<RValue> queue_id;
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
-                                                    add, &result));
+                                                add, &result));
     for (int i = 0; i < 100; ++i) {
         expected_result += i;
         RValue v(i);
@@ -126,6 +144,12 @@ TEST_F(ExecutionQueueTest, rvalue) {
     ASSERT_TRUE(stopped);
 }
 
+TEST_F(ExecutionQueueTest, rvalue) {
+    for (int i = 0; i < 2; ++i) {
+        test_rvalue(i);
+    }
+}
+
 struct PushArg {
     bthread::ExecutionQueueId<LongIntTask> id {0};
     butil::atomic<int64_t> total_num {0};
@@ -182,10 +206,16 @@ void* push_thread_which_addresses_execq(void *arg) {
     return NULL;
 }
 
-TEST_F(ExecutionQueueTest, performance) {
+void test_performance(bool use_pthread) {
     pthread_t threads[8];
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     int64_t result = 0;
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 add, &result));
@@ -207,7 +237,7 @@ TEST_F(ExecutionQueueTest, performance) {
     ProfilerStop();
     ASSERT_EQ(0, bthread::execution_queue_join(queue_id));
     ASSERT_EQ(pa.expected_value.load(), result);
-    LOG(INFO) << "With addressed execq, each execution_queue_execute takes " 
+    LOG(INFO) << "With addressed execq, each execution_queue_execute takes "
               << pa.total_time.load() / pa.total_num.load()
               << " total_num=" << pa.total_num
               << " ns with " << ARRAY_SIZE(threads) << " threads";
@@ -233,13 +263,19 @@ TEST_F(ExecutionQueueTest, performance) {
     ProfilerStop();
     ASSERT_EQ(0, bthread::execution_queue_join(queue_id));
     ASSERT_EQ(pa.expected_value.load(), result);
-    LOG(INFO) << "With id explicitly, execution_queue_execute takes " 
+    LOG(INFO) << "With id explicitly, execution_queue_execute takes "
               << pa.total_time.load() / pa.total_num.load()
               << " total_num=" << pa.total_num
               << " ns with " << ARRAY_SIZE(threads) << " threads";
 #endif  // BENCHMARK_BOTH
 }
 
+TEST_F(ExecutionQueueTest, performance) {
+    for (int i = 0; i < 2; ++i) {
+        test_performance(i);
+    }
+}
+
 volatile bool g_suspending = false;
 volatile bool g_should_be_urgent = false;
 int urgent_times = 0;
@@ -277,11 +313,17 @@ int add_with_suspend(void* meta, bthread::TaskIterator<LongIntTask>& iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, execute_urgent) {
+void test_execute_urgent(bool use_pthread) {
     g_should_be_urgent = false;
     pthread_t threads[10];
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     int64_t result = 0;
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 add_with_suspend, &result));
@@ -304,7 +346,7 @@ TEST_F(ExecutionQueueTest, execute_urgent) {
             usleep(100);
         }
         ASSERT_EQ(0, bthread::execution_queue_execute(
-                      queue_id, -1, &bthread::TASK_OPTIONS_URGENT));
+            queue_id, -1, &bthread::TASK_OPTIONS_URGENT));
         g_suspending = false;
         usleep(100);
     }
@@ -319,11 +361,23 @@ TEST_F(ExecutionQueueTest, execute_urgent) {
     ASSERT_EQ(pa.expected_value.load(), result);
 }
 
-TEST_F(ExecutionQueueTest, urgent_task_is_the_last_task) {
+TEST_F(ExecutionQueueTest, execute_urgent) {
+    for (int i = 0; i < 2; ++i) {
+        test_execute_urgent(i);
+    }
+}
+
+void test_urgent_task_is_the_last_task(bool use_pthread) {
     g_should_be_urgent = false;
     g_suspending = false;
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     int64_t result = 0;
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 add_with_suspend, &result));
@@ -334,11 +388,12 @@ TEST_F(ExecutionQueueTest, urgent_task_is_the_last_task) {
     }
     LOG(INFO) << "Going to push";
     int64_t expected = 0;
-    for (int i = 1; i < 100; ++i) {
-        expected += i;
-        ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, i));
+    for (int j = 1; j < 100; ++j) {
+        expected += j;
+        ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, j));
     }
-    ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, -1, &bthread::TASK_OPTIONS_URGENT));
+    ASSERT_EQ(0, bthread::execution_queue_execute(
+        queue_id, -1, &bthread::TASK_OPTIONS_URGENT));
     usleep(100);
     g_suspending = false;
     butil::atomic_thread_fence(butil::memory_order_acq_rel);
@@ -348,6 +403,11 @@ TEST_F(ExecutionQueueTest, urgent_task_is_the_last_task) {
     ASSERT_EQ(0, bthread::execution_queue_join(queue_id));
     ASSERT_EQ(expected, result);
 }
+TEST_F(ExecutionQueueTest, urgent_task_is_the_last_task) {
+    for (int i = 0; i < 2; ++i) {
+        test_urgent_task_is_the_last_task(i);
+    }
+}
 
 long next_task[1024];
 butil::atomic<int> num_threads(0);
@@ -376,11 +436,17 @@ int check_order(void* meta, bthread::TaskIterator<LongIntTask>& iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, multi_threaded_order) {
+void test_multi_threaded_order(bool use_pthread) {
     memset(next_task, 0, sizeof(next_task));
     long disorder_times = 0;
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 check_order, &disorder_times));
     pthread_t threads[12];
@@ -395,6 +461,12 @@ TEST_F(ExecutionQueueTest, multi_threaded_order) {
     ASSERT_EQ(0, disorder_times);
 }
 
+TEST_F(ExecutionQueueTest, multi_threaded_order) {
+    for (int i = 0; i < 2; ++i) {
+        test_multi_threaded_order(i);
+    }
+}
+
 int check_running_thread(void* arg, bthread::TaskIterator<LongIntTask>& iter) {
     if (iter.is_queue_stopped()) {
         return 0;
@@ -404,19 +476,31 @@ int check_running_thread(void* arg, bthread::TaskIterator<LongIntTask>& iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, in_place_task) {
+void test_in_place_task(bool use_pthread) {
     pthread_t thread_id = pthread_self();
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
-                                                check_running_thread, 
+                                                check_running_thread,
                                                 (void*)thread_id));
     ASSERT_EQ(0, bthread::execution_queue_execute(
-                queue_id, 0, &bthread::TASK_OPTIONS_INPLACE));
+        queue_id, 0, &bthread::TASK_OPTIONS_INPLACE));
     ASSERT_EQ(0, bthread::execution_queue_stop(queue_id));
     ASSERT_EQ(0, bthread::execution_queue_join(queue_id));
 }
 
+TEST_F(ExecutionQueueTest, in_place_task) {
+    for (int i = 0; i < 2; ++i) {
+        test_in_place_task(i);
+    }
+}
+
 struct InPlaceTask {
     bool first_task;
     pthread_t thread_id;
@@ -427,8 +511,8 @@ void *run_first_tasks(void* arg) {
     InPlaceTask task;
     task.first_task = true;
     task.thread_id = pthread_self();
-    EXPECT_EQ(0, bthread::execution_queue_execute(queue_id, task, 
-                                                  &bthread::TASK_OPTIONS_INPLACE));
+    EXPECT_EQ(0, bthread::execution_queue_execute(
+        queue_id, task, &bthread::TASK_OPTIONS_INPLACE));
     return NULL;
 }
 
@@ -455,12 +539,18 @@ int stuck_and_check_running_thread(void* arg, bthread::TaskIterator<InPlaceTask>
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, should_start_new_thread_on_more_tasks) {
+void test_should_start_new_thread_on_more_tasks(bool use_pthread) {
     bthread::ExecutionQueueId<InPlaceTask> queue_id = { 0 };
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     butil::atomic<int> futex(0);
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
-                                                stuck_and_check_running_thread, 
+                                                stuck_and_check_running_thread,
                                                 (void*)&futex));
     pthread_t thread;
     ASSERT_EQ(0, pthread_create(&thread, NULL, run_first_tasks, (void*)queue_id.value));
@@ -471,8 +561,8 @@ TEST_F(ExecutionQueueTest, should_start_new_thread_on_more_tasks) {
         InPlaceTask task;
         task.first_task = false;
         task.thread_id = pthread_self();
-        ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, task, 
-                                                      &bthread::TASK_OPTIONS_INPLACE));
+        ASSERT_EQ(0, bthread::execution_queue_execute(
+            queue_id, task, &bthread::TASK_OPTIONS_INPLACE));
     }
     futex.store(2);
     bthread::futex_wake_private(&futex, 1);
@@ -480,22 +570,34 @@ TEST_F(ExecutionQueueTest, should_start_new_thread_on_more_tasks) {
     ASSERT_EQ(0, bthread::execution_queue_join(queue_id));
 }
 
+TEST_F(ExecutionQueueTest, should_start_new_thread_on_more_tasks) {
+    for (int i = 0; i < 2; ++i) {
+        test_should_start_new_thread_on_more_tasks(i);
+    }
+}
+
 void* inplace_push_thread(void* arg) {
     bthread::ExecutionQueueId<LongIntTask> id = { (uint64_t)arg };
     int thread_id = num_threads.fetch_add(1, butil::memory_order_relaxed);
     LOG(INFO) << "Start thread" << thread_id;
     for (int i = 0; i < 100000; ++i) {
         bthread::execution_queue_execute(id, ((long)thread_id << 32) | i,
-                            &bthread::TASK_OPTIONS_INPLACE);
+                                         &bthread::TASK_OPTIONS_INPLACE);
     }
     return NULL;
 }
 
-TEST_F(ExecutionQueueTest, inplace_and_order) {
+void test_inplace_and_order(bool use_pthread) {
     memset(next_task, 0, sizeof(next_task));
     long disorder_times = 0;
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 check_order, &disorder_times));
     pthread_t threads[12];
@@ -510,6 +612,12 @@ TEST_F(ExecutionQueueTest, inplace_and_order) {
     ASSERT_EQ(0, disorder_times);
 }
 
+TEST_F(ExecutionQueueTest, inplace_and_order) {
+    for (int i = 0; i < 2; ++i) {
+        test_inplace_and_order(i);
+    }
+}
+
 TEST_F(ExecutionQueueTest, size_of_task_node) {
     LOG(INFO) << "sizeof(TaskNode)=" << sizeof(bthread::TaskNode);
 }
@@ -535,9 +643,15 @@ int add_with_suspend2(void* meta, bthread::TaskIterator<LongIntTask>& iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, cancel) {
+void test_cancel(bool use_pthread) {
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     int64_t result = 0;
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 add_with_suspend2, &result));
@@ -559,6 +673,12 @@ TEST_F(ExecutionQueueTest, cancel) {
     ASSERT_EQ(0, result);
 }
 
+TEST_F(ExecutionQueueTest, cancel) {
+    for (int i = 0; i < 2; ++i) {
+        test_cancel(i);
+    }
+}
+
 struct CancelSelf {
     butil::atomic<bthread::TaskHandle*> handle;
 };
@@ -576,9 +696,15 @@ int cancel_self(void* /*meta*/, bthread::TaskIterator<CancelSelf*>& iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, cancel_self) {
+void test_cancel_self(bool use_pthread) {
     bthread::ExecutionQueueId<CancelSelf*> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 cancel_self, NULL));
     CancelSelf task;
@@ -590,6 +716,12 @@ TEST_F(ExecutionQueueTest, cancel_self) {
     ASSERT_EQ(0, bthread::execution_queue_join(queue_id));
 }
 
+TEST_F(ExecutionQueueTest, cancel_self) {
+    for (int i = 0; i < 2; ++i) {
+        test_cancel_self(i);
+    }
+}
+
 struct AddTask {
     int value;
     bool cancel_task;
@@ -628,7 +760,7 @@ int add_with_cancel(void* meta, bthread::TaskIterator<AddTask>& iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, random_cancel) {
+void test_random_cancel(bool use_pthread) {
     bthread::ExecutionQueueId<AddTask> queue_id = { 0 };
     AddMeta m;
     m.sum = 0;
@@ -660,8 +792,8 @@ TEST_F(ExecutionQueueTest, random_cancel) {
             t.cancel_task = true;
             t.cancel_value = i;
             t.handle = h;
-            ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, t, 
-                                    &bthread::TASK_OPTIONS_URGENT));
+            ASSERT_EQ(0, bthread::execution_queue_execute(
+                queue_id, t, &bthread::TASK_OPTIONS_URGENT));
         } else {
             // do nothing;
         }
@@ -673,6 +805,12 @@ TEST_F(ExecutionQueueTest, random_cancel) {
     LOG(INFO) << "sum=" << m.sum << " race_times=" << m.race_times
               << " succ_times=" << m.succ_times
               << " fail_times=" << m.fail_times;
+}
+
+TEST_F(ExecutionQueueTest, random_cancel) {
+    for (int i = 0; i < 2; ++i) {
+        test_random_cancel(i);
+    }
 
 }
 
@@ -685,13 +823,19 @@ int add2(void* meta, bthread::TaskIterator<LongIntTask> &iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, not_do_iterate_at_all) {
+void test_not_do_iterate_at_all(bool use_pthread) {
     int64_t result = 0;
     int64_t expected_result = 0;
     bthread::ExecutionQueueId<LongIntTask> queue_id;
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
-                                                    add2, &result));
+                                                add2, &result));
     for (int i = 0; i < 100; ++i) {
         expected_result += i;
         ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, i));
@@ -702,6 +846,12 @@ TEST_F(ExecutionQueueTest, not_do_iterate_at_all) {
     ASSERT_EQ(expected_result, result);
 }
 
+TEST_F(ExecutionQueueTest, not_do_iterate_at_all) {
+    for (int i = 0; i < 2; ++i) {
+        test_not_do_iterate_at_all(i);
+    }
+}
+
 int add_with_suspend3(void* meta, bthread::TaskIterator<LongIntTask>& iter) {
     int64_t* result = (int64_t*)meta;
     if (iter.is_queue_stopped()) {
@@ -723,10 +873,16 @@ int add_with_suspend3(void* meta, bthread::TaskIterator<LongIntTask>& iter) {
     return 0;
 }
 
-TEST_F(ExecutionQueueTest, cancel_unexecuted_high_priority_task) {
+void test_cancel_unexecuted_high_priority_task(bool use_pthread) {
     g_should_be_urgent = false;
     bthread::ExecutionQueueId<LongIntTask> queue_id = { 0 }; // to suppress warnings
     bthread::ExecutionQueueOptions options;
+    options.use_pthread = use_pthread;
+    if (options.use_pthread) {
+        LOG(INFO) << "================ pthread ================";
+    } else {
+        LOG(INFO) << "================ bthread ================";
+    }
     int64_t result = 0;
     ASSERT_EQ(0, bthread::execution_queue_start(&queue_id, &options,
                                                 add_with_suspend3, &result));
@@ -740,11 +896,11 @@ TEST_F(ExecutionQueueTest, cancel_unexecuted_high_priority_task) {
     // expecting that both operations are successful.
     bthread::TaskHandle h;
     ASSERT_EQ(0, bthread::execution_queue_execute(
-                        queue_id, -100, &bthread::TASK_OPTIONS_URGENT, &h));
+        queue_id, -100, &bthread::TASK_OPTIONS_URGENT, &h));
     ASSERT_EQ(0, bthread::execution_queue_cancel(h));
-    
+
     // Resume executor
-    g_suspending = false;  
+    g_suspending = false;
 
     // Push a normal task
     ASSERT_EQ(0, bthread::execution_queue_execute(queue_id, 12345));
@@ -755,4 +911,10 @@ TEST_F(ExecutionQueueTest, cancel_unexecuted_high_priority_task) {
 
     ASSERT_EQ(12345, result);
 }
+
+TEST_F(ExecutionQueueTest, cancel_unexecuted_high_priority_task) {
+    for (int i = 0; i < 2; ++i) {
+        test_cancel_unexecuted_high_priority_task(i);
+    }
+}
 } // namespace
