@@ -16,12 +16,19 @@
 // under the License.
 
 #include <gtest/gtest.h>
+#include <butil/fd_guard.h>
+#include <butil/fd_utility.h>
 #include "butil/errno.h"
 #include "butil/endpoint.h"
 #include "butil/logging.h"
 #include "butil/containers/flat_map.h"
 #include "butil/details/extended_endpoint.hpp"
 
+namespace butil {
+int pthread_timed_connect(int sockfd, const struct sockaddr* serv_addr,
+                          socklen_t addrlen, const timespec* abstime);
+}
+
 namespace {
 
 using butil::details::ExtendedEndPoint;
@@ -472,4 +479,53 @@ TEST(EndPointTest, endpoint_concurrency) {
     }
 }
 
+const char* g_hostname = "baidu.com";
+
+TEST(EndPointTest, tcp_connect) {
+    butil::EndPoint ep;
+    ASSERT_EQ(0, butil::hostname2endpoint(g_hostname, 80, &ep));
+    {
+        butil::fd_guard sockfd(butil::tcp_connect(ep, NULL));
+        ASSERT_LE(0, sockfd);
+    }
+    {
+        butil::fd_guard sockfd(butil::tcp_connect(ep, NULL, 1000));
+        ASSERT_LE(0, sockfd);
+    }
+    {
+        butil::fd_guard sockfd(butil::tcp_connect(ep, NULL, 1));
+        ASSERT_EQ(-1, sockfd);
+        ASSERT_EQ(ETIMEDOUT, errno);
+    }
+
+    {
+        struct sockaddr_storage serv_addr{};
+        socklen_t serv_addr_size = 0;
+        ASSERT_EQ(0, endpoint2sockaddr(ep, &serv_addr, &serv_addr_size));
+        butil::fd_guard sockfd(socket(serv_addr.ss_family, SOCK_STREAM, 0));
+        ASSERT_LE(0, sockfd);
+        bool is_blocking = butil::is_blocking(sockfd);
+        ASSERT_EQ(0, butil::pthread_timed_connect(
+            sockfd, (struct sockaddr*) &serv_addr, serv_addr_size, NULL));
+        ASSERT_EQ(is_blocking, butil::is_blocking(sockfd));
+    }
+
+    {
+        struct sockaddr_storage serv_addr{};
+        socklen_t serv_addr_size = 0;
+        ASSERT_EQ(0, endpoint2sockaddr(ep, &serv_addr, &serv_addr_size));
+        butil::fd_guard sockfd(socket(serv_addr.ss_family, SOCK_STREAM, 0));
+        ASSERT_LE(0, sockfd);
+        bool is_blocking = butil::is_blocking(sockfd);
+        // In most cases, 1 millisecond will result in a connection timeout.
+        timespec abstime = butil::milliseconds_from_now(1);
+        const int rc = butil::pthread_timed_connect(
+            sockfd, (struct sockaddr*) &serv_addr,
+            serv_addr_size, &abstime);
+        ASSERT_EQ(-1, rc);
+        ASSERT_EQ(ETIMEDOUT, errno);
+        ASSERT_EQ(is_blocking, butil::is_blocking(sockfd));
+    }
+}
+
 } // end of namespace
